@@ -6,7 +6,7 @@ use 5.014;
 
 use parent 'Class::Accessor';
 
-our $VERSION = '4.19';
+our $VERSION = '5.00';
 
 Travel::Status::DE::HAFAS::Message->mk_ro_accessors(
 	qw(short type text code prio is_him ref_count));
@@ -69,7 +69,7 @@ Travel::Status::DE::HAFAS::Message - An arrival/departure-related message.
 
 =head1 VERSION
 
-version 4.19
+version 5.00
 
 =head1 DESCRIPTION
 
@@ -112,7 +112,7 @@ towards [destination]".
 
 =item C
 
-"Current information available", possibly more.
+"Current information available", "Journey cancelled", "connection may not be caught", possibly more.
 
 =item D
 
