@@ -10,7 +10,7 @@ use Encode qw(decode);
 use JSON;
 use LWP::UserAgent;
 
-our $VERSION = '4.19';
+our $VERSION = '5.00';
 
 # {{{ Constructors
 
@@ -180,7 +180,7 @@ finder services
 
 =head1 VERSION
 
-version 4.19
+version 5.00
 
 =head1 DESCRIPTION
 
@@ -191,6 +191,10 @@ available at L<https://reiseauskunft.bahn.de/bin/ajax-getstop.exe/dn>.
 It takes a string (usually a location or station name) and reports all
 stations and stops which are lexically similar to it.
 
+StopFinder typically gives less coarse results than
+Travel::Status::DE::HAFAS(3pm)'s locationSearch method. However, it is unclear
+whether HAFAS instances will continue supporting it in the future.
+
 =head1 METHODS
 
 =over
